searchNodes=[{"type":"module","title":"Grains","doc":"Grains describes data flow as a graph with interchangeable parts. To accomplish that grains divides the graph ( Recipe ) from the implementation of the nodes ( Grains ). Together these parts can be used to make a Bread which describes how the graph translates to processes. Each process has a symbolic short name, which is resolved to the registered name of the process via a map ( %Bread{}.process_map ).","ref":"Grains.html"},{"type":"function","title":"Grains.combine/2","doc":"Combine two sets of recipes and grains.","ref":"Grains.html#combine/2"},{"type":"function","title":"Grains.concat_name/2","doc":"","ref":"Grains.html#concat_name/2"},{"type":"function","title":"Grains.debug_pull_chain/3","doc":"Debug multiple asynchronous pull paths. This function floods pulls upwards from first_grain towards last_grain . The function returns {:ok, number_of_paths} to indicate how often message should be received by the caller of this function.","ref":"Grains.html#debug_pull_chain/3"},{"type":"function","title":"Grains.debug_reply_chain/2","doc":"Debug an asynchronous data chain. The function expects a list of grains. The grain currently handling the message checks if its name matches the first element of the list. If so, it pops that element and attempts to send the message to the next grain in the list. If only one element in the list remains, and that element matches the name of the current grain handling the list, this grain replies to the sender with message .","ref":"Grains.html#debug_reply_chain/2"},{"type":"function","title":"Grains.debug_reply_chain/3","doc":"Debug multiple asynchronous data paths. This function floods message along all paths between first_grain and last_grain . It is a generalization of debug_reply_chain/2 , which does not require specifying the paths explicitly. The function returns {:ok, number_of_paths} to indicate how often message should be received by the caller of this function.","ref":"Grains.html#debug_reply_chain/3"},{"type":"function","title":"Grains.get_name/2","doc":"","ref":"Grains.html#get_name/2"},{"type":"function","title":"Grains.get_original_recipe!/1","doc":"See Grains.Supervisor.get_original_recipe!/1 .","ref":"Grains.html#get_original_recipe!/1"},{"type":"function","title":"Grains.get_substate/1","doc":"Retrieve a grain's substate. This is similar to :sys.get_state/1 . Note that this function should usually not be used, but it can be useful for debugging or testing. Errors The function exits the caller on error. For example, if no bread with the supplied id can be found, this results in a no_proc exit.","ref":"Grains.html#get_substate/1"},{"type":"function","title":"Grains.ghost/2","doc":"A ghost edge. A ghost edge is an edge from one grain through another which is not usable by Grains itself, but which may indicate communication between the two nodes using standard tools such as GenServer.call/2 .","ref":"Grains.html#ghost/2"},{"type":"function","title":"Grains.make_bread_name/2","doc":"","ref":"Grains.html#make_bread_name/2"},{"type":"function","title":"Grains.make_name/3","doc":"","ref":"Grains.html#make_name/3"},{"type":"function","title":"Grains.merge/2","doc":"Merges two grain maps into one. The function works the same as Map.merge/2 : all grains in the second argument are added to the first, overriding any existing one.","ref":"Grains.html#merge/2"},{"type":"function","title":"Grains.merge_recipes/3","doc":"See Grains.Recipe.merge/3 .","ref":"Grains.html#merge_recipes/3"},{"type":"function","title":"Grains.new/1","doc":"","ref":"Grains.html#new/1"},{"type":"function","title":"Grains.new_recipe/2","doc":"See Grains.Recipe.new/2 .","ref":"Grains.html#new_recipe/2"},{"type":"function","title":"Grains.periodic/3","doc":"Description of a periodic grain.","ref":"Grains.html#periodic/3"},{"type":"macro","title":"Grains.route/1","doc":"Convenient version of routes/2 . Takes a list of tuples of the form [{pattern, grains}] .","ref":"Grains.html#route/1"},{"type":"macro","title":"Grains.route/2","doc":"Define a pattern which messages must match in order to be pushed to the successor grains. Every message gets tested against the pattern before it is pushed to the successors.","ref":"Grains.html#route/2"},{"type":"function","title":"Grains.start_supervised/3","doc":"See Grains.Supervisor.start_link/3 .","ref":"Grains.html#start_supervised/3"},{"type":"module","title":"Grains.Bread","doc":"A %Bread{} is the compiled version of Recipe plus Grains that allow the Supervisor to run the processes and wire them correctly. When defining the node implementations an optional startup order can be passed as the last parameter: Grains.new(%{ A =&gt; { ImplA , [ ] , [ ] , 1 } , B =&gt; { ImplB , [ ] , [ ] , 2 } , C =&gt; { ImplC , [ ] , [ ] , 3 } } ) This will create child specs of the given processes in ascending order.","ref":"Grains.Bread.html"},{"type":"function","title":"Grains.Bread.bake/3","doc":"Takes a recipe, grains and optionally a map of default implementations and returns a bread. Does not start any processes yet. Optional Arguments :id Set a custom bread id","ref":"Grains.Bread.html#bake/3"},{"type":"function","title":"Grains.Bread.check/1","doc":"","ref":"Grains.Bread.html#check/1"},{"type":"function","title":"Grains.Bread.gen_id/0","doc":"Generates a random atom to be used as an id for the Bread","ref":"Grains.Bread.html#gen_id/0"},{"type":"function","title":"Grains.Bread.ghosts/2","doc":"Retrieve all ghosts of short_name .","ref":"Grains.Bread.html#ghosts/2"},{"type":"function","title":"Grains.Bread.paths_between/3","doc":"Retrieve all cycle-free paths between first and last .","ref":"Grains.Bread.html#paths_between/3"},{"type":"function","title":"Grains.Bread.predecessors/2","doc":"Retrieve all predecessors of short_name .","ref":"Grains.Bread.html#predecessors/2"},{"type":"function","title":"Grains.Bread.pull_paths_between/3","doc":"Retrieve all cycle-free pull-paths between first and last .","ref":"Grains.Bread.html#pull_paths_between/3"},{"type":"function","title":"Grains.Bread.routes/2","doc":"Retrieve all routes of short_name .","ref":"Grains.Bread.html#routes/2"},{"type":"function","title":"Grains.Bread.successors/2","doc":"Retrieve all successors of short_name .","ref":"Grains.Bread.html#successors/2"},{"type":"type","title":"Grains.Bread.t/0","doc":"","ref":"Grains.Bread.html#t:t/0"},{"type":"module","title":"Grains.DebugTimer","doc":"Debug version of Grains.Timer . To replace the normal Timer with this, start you bread like this: recipe = Recipe . new ( :Periodic , %{ Source =&gt; Grains . periodic ( Sink , 10 , %{ with_timestamps : true } ) , } ) grains = Grains . new ( %{ Source =&gt; { Source , %{ } , [ ] } , Sink =&gt; { Sink , %{ } , [ ] } , Source.Timer.Sink =&gt; { Grains.DebugTimer , [ ] , [ ] } } ) { :ok , bread } = Grains.Supervisor . start_link ( recipe , grains , id : Test ) debug_timer = Grains . get_name ( bread , Source.Timer.Sink ) send ( debug_timer , :tick ) With Timestamps Similarly to Grains.Timer , the debug timer accepts an option with_timestamps . If set to true, the :tick must include a timestamp, which is then used when forwarding the received message downstream. If with_timestamps is not set, the :tick must not include a timestamp, and the message is forwarded as-is. With Tags The debug timer also accepts an option with_tag . If set to an atom, the timer will use pull_with_tag instead of pull .","ref":"Grains.DebugTimer.html"},{"type":"function","title":"Grains.DebugTimer.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.DebugTimer.html#child_spec/1"},{"type":"function","title":"Grains.DebugTimer.handle_pull/2","doc":"Callback implementation for Grains.GenGrain.handle_pull/2 .","ref":"Grains.DebugTimer.html#handle_pull/2"},{"type":"function","title":"Grains.DebugTimer.handle_pull/3","doc":"Callback implementation for Grains.GenGrain.handle_pull/3 .","ref":"Grains.DebugTimer.html#handle_pull/3"},{"type":"function","title":"Grains.DebugTimer.handle_push/3","doc":"Callback implementation for Grains.GenGrain.handle_push/3 .","ref":"Grains.DebugTimer.html#handle_push/3"},{"type":"function","title":"Grains.DebugTimer.init/1","doc":"Callback implementation for GenServer.init/1 .","ref":"Grains.DebugTimer.html#init/1"},{"type":"function","title":"Grains.DebugTimer.name/2","doc":"See Grains.Timer.name/2 .","ref":"Grains.DebugTimer.html#name/2"},{"type":"function","title":"Grains.DebugTimer.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.DebugTimer.html#own_full_name/0"},{"type":"function","title":"Grains.DebugTimer.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.DebugTimer.html#own_name/0"},{"type":"function","title":"Grains.DebugTimer.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.DebugTimer.html#pull/0"},{"type":"function","title":"Grains.DebugTimer.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.DebugTimer.html#pull/1"},{"type":"function","title":"Grains.DebugTimer.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.DebugTimer.html#pull_with_tag/1"},{"type":"function","title":"Grains.DebugTimer.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.DebugTimer.html#pull_with_tag/2"},{"type":"function","title":"Grains.DebugTimer.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.DebugTimer.html#push/1"},{"type":"function","title":"Grains.DebugTimer.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.DebugTimer.html#push/2"},{"type":"module","title":"Grains.DebugTimer.State","doc":"","ref":"Grains.DebugTimer.State.html"},{"type":"behaviour","title":"Grains.GenGrain","doc":"","ref":"Grains.GenGrain.html"},{"type":"function","title":"Grains.GenGrain.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.GenGrain.html#child_spec/1"},{"type":"function","title":"Grains.GenGrain.format_status/2","doc":"Callback implementation for GenServer.format_status/2 .","ref":"Grains.GenGrain.html#format_status/2"},{"type":"function","title":"Grains.GenGrain.full_name/1","doc":"","ref":"Grains.GenGrain.html#full_name/1"},{"type":"function","title":"Grains.GenGrain.ghosts/0","doc":"Get a list of the grain's ghosts. Note that this can only be called from a grain process.","ref":"Grains.GenGrain.html#ghosts/0"},{"type":"function","title":"Grains.GenGrain.ghosts_full_name/1","doc":"Retrieve ghost's ghost_short_name full name. Note that this can only be called from a grain process.","ref":"Grains.GenGrain.html#ghosts_full_name/1"},{"type":"function","title":"Grains.GenGrain.handle_continue/2","doc":"Callback implementation for GenServer.handle_continue/2 .","ref":"Grains.GenGrain.html#handle_continue/2"},{"type":"callback","title":"Grains.GenGrain.handle_pull/2","doc":"Handle a pull by a grain. When pulled by pull/0,1 , this callback is triggered. The grain directly replies with a :reply tuple, ignores the pull with :noreply , or use push/1,2 and :noreply . See handle_pull/3 for handling pull_with_tag/1,2 .","ref":"Grains.GenGrain.html#c:handle_pull/2"},{"type":"callback","title":"Grains.GenGrain.handle_pull/3","doc":"Handle a tagged pull by a grain. When pulled by pull_with_tag/1,2 , this callback is triggered. The grain directly replies with a :reply tuple, ignores the pull with :noreply , or use push/1,2 and :noreply . See handle_pull/2 for handling pull/0,1 .","ref":"Grains.GenGrain.html#c:handle_pull/3"},{"type":"callback","title":"Grains.GenGrain.handle_push/3","doc":"Handle incoming data. This callback is triggered when any grain pushes data to this grain, using either push/1 or push/2 .","ref":"Grains.GenGrain.html#c:handle_push/3"},{"type":"function","title":"Grains.GenGrain.init/1","doc":"Callback implementation for GenServer.init/1 .","ref":"Grains.GenGrain.html#init/1"},{"type":"function","title":"Grains.GenGrain.own_full_name/0","doc":"","ref":"Grains.GenGrain.html#own_full_name/0"},{"type":"function","title":"Grains.GenGrain.own_name/0","doc":"","ref":"Grains.GenGrain.html#own_name/0"},{"type":"function","title":"Grains.GenGrain.predecessors/0","doc":"Get a list of the grain's predecessors. Note that this can only be called from a grain process.","ref":"Grains.GenGrain.html#predecessors/0"},{"type":"function","title":"Grains.GenGrain.pull/0","doc":"","ref":"Grains.GenGrain.html#pull/0"},{"type":"function","title":"Grains.GenGrain.pull/1","doc":"","ref":"Grains.GenGrain.html#pull/1"},{"type":"function","title":"Grains.GenGrain.pull_with_tag/1","doc":"","ref":"Grains.GenGrain.html#pull_with_tag/1"},{"type":"function","title":"Grains.GenGrain.pull_with_tag/2","doc":"","ref":"Grains.GenGrain.html#pull_with_tag/2"},{"type":"function","title":"Grains.GenGrain.push/1","doc":"","ref":"Grains.GenGrain.html#push/1"},{"type":"function","title":"Grains.GenGrain.push/2","doc":"","ref":"Grains.GenGrain.html#push/2"},{"type":"function","title":"Grains.GenGrain.start_link/5","doc":"","ref":"Grains.GenGrain.html#start_link/5"},{"type":"function","title":"Grains.GenGrain.successors/0","doc":"Get a list of the grain's successors. Note that this can only be called from a grain process.","ref":"Grains.GenGrain.html#successors/0"},{"type":"type","title":"Grains.GenGrain.from/0","doc":"","ref":"Grains.GenGrain.html#t:from/0"},{"type":"type","title":"Grains.GenGrain.reply/0","doc":"","ref":"Grains.GenGrain.html#t:reply/0"},{"type":"type","title":"Grains.GenGrain.return/0","doc":"","ref":"Grains.GenGrain.html#t:return/0"},{"type":"type","title":"Grains.GenGrain.state/0","doc":"","ref":"Grains.GenGrain.html#t:state/0"},{"type":"type","title":"Grains.GenGrain.tag/0","doc":"","ref":"Grains.GenGrain.html#t:tag/0"},{"type":"module","title":"Grains.Recipe","doc":"A Recipe describes the data flow between the processes.","ref":"Grains.Recipe.html"},{"type":"function","title":"Grains.Recipe.merge/3","doc":"Merges to recipes by combining all edges.","ref":"Grains.Recipe.html#merge/3"},{"type":"function","title":"Grains.Recipe.new/2","doc":"","ref":"Grains.Recipe.html#new/2"},{"type":"function","title":"Grains.Recipe.to_mermaid/2","doc":"Returns a mermaid flowchart as a string. Arguments to_mermaid/2 supports an optional args argument. The following options are supported: type : Graph type as string, can be either &quot;flowchart&quot; or &quot;graph&quot; (default: &quot;graph&quot; ) direction : Flowchart orientation (default: &quot;TD&quot; ) merge_edges : Merge multiple edges between a predecessor and a successor (default: false )","ref":"Grains.Recipe.html#to_mermaid/2"},{"type":"type","title":"Grains.Recipe.t/0","doc":"","ref":"Grains.Recipe.html#t:t/0"},{"type":"module","title":"Grains.Supervisor","doc":"","ref":"Grains.Supervisor.html"},{"type":"function","title":"Grains.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Supervisor.html#child_spec/1"},{"type":"function","title":"Grains.Supervisor.get_bread_supervisor/1","doc":"Retrieve the bread supervisor. Crashes if :bread_supervisor is not started.","ref":"Grains.Supervisor.html#get_bread_supervisor/1"},{"type":"function","title":"Grains.Supervisor.get_original_recipe!/1","doc":"Get the original recipe underlying the supervisor .","ref":"Grains.Supervisor.html#get_original_recipe!/1"},{"type":"function","title":"Grains.Supervisor.get_root_name/1","doc":"Retrieve the root bread name. This function assumes that the supervisor has a registered name.","ref":"Grains.Supervisor.html#get_root_name/1"},{"type":"function","title":"Grains.Supervisor.init/1","doc":"Callback implementation for Supervisor.init/1 .","ref":"Grains.Supervisor.html#init/1"},{"type":"function","title":"Grains.Supervisor.start_link/3","doc":"","ref":"Grains.Supervisor.html#start_link/3"},{"type":"function","title":"Grains.Supervisor.which_grains/1","doc":"Retrieve the id and registered name of currently running grains.","ref":"Grains.Supervisor.html#which_grains/1"},{"type":"type","title":"Grains.Supervisor.t/0","doc":"","ref":"Grains.Supervisor.html#t:t/0"},{"type":"module","title":"Grains.Support","doc":"Support functions to simplify testing.","ref":"Grains.Support.html"},{"type":"function","title":"Grains.Support.push_into_test/2","doc":"Publish a message to the &quot;Grain under Test&quot; Example iex&gt; test_setup = setup_test! ( Grains.Support.Cache , [ ] ) iex&gt; test_setup |&gt; push_into_test ( 1 ) |&gt; push_into_test ( 2 ) iex&gt; assert_receive 1 1 iex&gt; assert_receive 2 2","ref":"Grains.Support.html#push_into_test/2"},{"type":"function","title":"Grains.Support.setup_test!/2","doc":"Generate boiler plate to blackbox-test a grain. This helper starts a supervised setup to test a &quot;Grain under Test&quot; (GuT): Publisher -- -&gt; GuT -- -&gt; Subscriber By publishing messages through the Publisher , GuT can be blackbox-tested: the Subscriber forwards all messages to the process that called setup_test!/2 . Use push_into_test/2 to publish a message to GuT. Arguments module : The module of the GuT args : The arguments to initialize a GuT","ref":"Grains.Support.html#setup_test!/2"},{"type":"module","title":"Grains.Support.Cache","doc":"A grain to forward messages downstream and cache the last value it received. This grain is meant for testing purposes, where another grain is to be tested.","ref":"Grains.Support.Cache.html"},{"type":"function","title":"Grains.Support.Cache.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Support.Cache.html#child_spec/1"},{"type":"function","title":"Grains.Support.Cache.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.Support.Cache.html#own_full_name/0"},{"type":"function","title":"Grains.Support.Cache.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.Support.Cache.html#own_name/0"},{"type":"function","title":"Grains.Support.Cache.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.Support.Cache.html#pull/0"},{"type":"function","title":"Grains.Support.Cache.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.Support.Cache.html#pull/1"},{"type":"function","title":"Grains.Support.Cache.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.Support.Cache.html#pull_with_tag/1"},{"type":"function","title":"Grains.Support.Cache.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.Support.Cache.html#pull_with_tag/2"},{"type":"function","title":"Grains.Support.Cache.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.Support.Cache.html#push/1"},{"type":"function","title":"Grains.Support.Cache.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.Support.Cache.html#push/2"},{"type":"module","title":"Grains.Support.Constant","doc":"A grain which holds a constant value.","ref":"Grains.Support.Constant.html"},{"type":"function","title":"Grains.Support.Constant.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Support.Constant.html#child_spec/1"},{"type":"function","title":"Grains.Support.Constant.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.Support.Constant.html#own_full_name/0"},{"type":"function","title":"Grains.Support.Constant.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.Support.Constant.html#own_name/0"},{"type":"function","title":"Grains.Support.Constant.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.Support.Constant.html#pull/0"},{"type":"function","title":"Grains.Support.Constant.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.Support.Constant.html#pull/1"},{"type":"function","title":"Grains.Support.Constant.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.Support.Constant.html#pull_with_tag/1"},{"type":"function","title":"Grains.Support.Constant.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.Support.Constant.html#pull_with_tag/2"},{"type":"function","title":"Grains.Support.Constant.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.Support.Constant.html#push/1"},{"type":"function","title":"Grains.Support.Constant.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.Support.Constant.html#push/2"},{"type":"module","title":"Grains.Support.Constant.State","doc":"","ref":"Grains.Support.Constant.State.html"},{"type":"module","title":"Grains.Support.Publisher","doc":"A grain to forward messages downstream. This grain is meant for testing purposes, where another grain is to be tested.","ref":"Grains.Support.Publisher.html"},{"type":"function","title":"Grains.Support.Publisher.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Support.Publisher.html#child_spec/1"},{"type":"function","title":"Grains.Support.Publisher.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.Support.Publisher.html#own_full_name/0"},{"type":"function","title":"Grains.Support.Publisher.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.Support.Publisher.html#own_name/0"},{"type":"function","title":"Grains.Support.Publisher.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.Support.Publisher.html#pull/0"},{"type":"function","title":"Grains.Support.Publisher.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.Support.Publisher.html#pull/1"},{"type":"function","title":"Grains.Support.Publisher.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.Support.Publisher.html#pull_with_tag/1"},{"type":"function","title":"Grains.Support.Publisher.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.Support.Publisher.html#pull_with_tag/2"},{"type":"function","title":"Grains.Support.Publisher.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.Support.Publisher.html#push/1"},{"type":"function","title":"Grains.Support.Publisher.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.Support.Publisher.html#push/2"},{"type":"module","title":"Grains.Support.Sink","doc":"A grain to eat incoming messages without any further processing. This grain is meant for testing purposes, where the incoming messages should be ignored. This is mainly useful when a grain is needed to satisfy the recipe, but the messages sent to this grain are actually of no consequence.","ref":"Grains.Support.Sink.html"},{"type":"function","title":"Grains.Support.Sink.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Support.Sink.html#child_spec/1"},{"type":"function","title":"Grains.Support.Sink.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.Support.Sink.html#own_full_name/0"},{"type":"function","title":"Grains.Support.Sink.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.Support.Sink.html#own_name/0"},{"type":"function","title":"Grains.Support.Sink.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.Support.Sink.html#pull/0"},{"type":"function","title":"Grains.Support.Sink.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.Support.Sink.html#pull/1"},{"type":"function","title":"Grains.Support.Sink.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.Support.Sink.html#pull_with_tag/1"},{"type":"function","title":"Grains.Support.Sink.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.Support.Sink.html#pull_with_tag/2"},{"type":"function","title":"Grains.Support.Sink.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.Support.Sink.html#push/1"},{"type":"function","title":"Grains.Support.Sink.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.Support.Sink.html#push/2"},{"type":"module","title":"Grains.Support.Subscriber","doc":"A grain to pull message and forward pushed messages to a subscribing process. This grain is meant for testing purposes, where the incoming messages should be forwarded to the test process.","ref":"Grains.Support.Subscriber.html"},{"type":"function","title":"Grains.Support.Subscriber.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Support.Subscriber.html#child_spec/1"},{"type":"function","title":"Grains.Support.Subscriber.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.Support.Subscriber.html#own_full_name/0"},{"type":"function","title":"Grains.Support.Subscriber.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.Support.Subscriber.html#own_name/0"},{"type":"function","title":"Grains.Support.Subscriber.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.Support.Subscriber.html#pull/0"},{"type":"function","title":"Grains.Support.Subscriber.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.Support.Subscriber.html#pull/1"},{"type":"function","title":"Grains.Support.Subscriber.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.Support.Subscriber.html#pull_with_tag/1"},{"type":"function","title":"Grains.Support.Subscriber.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.Support.Subscriber.html#pull_with_tag/2"},{"type":"function","title":"Grains.Support.Subscriber.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.Support.Subscriber.html#push/1"},{"type":"function","title":"Grains.Support.Subscriber.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.Support.Subscriber.html#push/2"},{"type":"module","title":"Grains.Support.Subscriber.State","doc":"","ref":"Grains.Support.Subscriber.State.html"},{"type":"module","title":"Grains.Support.Trace","doc":"This is a wrapper around :erlang.trace/3 adapted to trace the sending and receiving push/pull messages in grains. The Trace server has to be started before it can be used, either directly with Trace.start_link/1 or with start_supervised!(Trace) in tests.","ref":"Grains.Support.Trace.html"},{"type":"function","title":"Grains.Support.Trace.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Support.Trace.html#child_spec/1"},{"type":"function","title":"Grains.Support.Trace.init/1","doc":"Callback implementation for GenServer.init/1 .","ref":"Grains.Support.Trace.html#init/1"},{"type":"function","title":"Grains.Support.Trace.pull/2","doc":"Trace pull . Arguments :target : The shortname of the grain to-be-traced. :what : One of :receive or :send , indicating if sending or receiving should be traced. :tracer : (optional) tracer process, defaults to MODULE","ref":"Grains.Support.Trace.html#pull/2"},{"type":"function","title":"Grains.Support.Trace.push/2","doc":"Trace push . Arguments :target : The shortname of the grain to-be-traced. :what : One of :receive or :send , indicating if sending or receiving should be traced.","ref":"Grains.Support.Trace.html#push/2"},{"type":"function","title":"Grains.Support.Trace.start_link/1","doc":"","ref":"Grains.Support.Trace.html#start_link/1"},{"type":"function","title":"Grains.Support.Trace.stop_pull/2","doc":"Stop tracing pull . Arguments :target : The shortname of the grain to-be-traced. :what : One of :receive or :send , indicating if sending or receiving should be traced. :tracer : (optional) tracer process, defaults to MODULE","ref":"Grains.Support.Trace.html#stop_pull/2"},{"type":"function","title":"Grains.Support.Trace.stop_push/2","doc":"Stop tracing push . Arguments :target : The shortname of the grain to-be-traced. :what : One of :receive or :send , indicating if sending or receiving should be traced.","ref":"Grains.Support.Trace.html#stop_push/2"},{"type":"module","title":"Grains.Support.Transformer","doc":"A grain to transform messages and push them further along.","ref":"Grains.Support.Transformer.html"},{"type":"function","title":"Grains.Support.Transformer.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Support.Transformer.html#child_spec/1"},{"type":"function","title":"Grains.Support.Transformer.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.Support.Transformer.html#own_full_name/0"},{"type":"function","title":"Grains.Support.Transformer.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.Support.Transformer.html#own_name/0"},{"type":"function","title":"Grains.Support.Transformer.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.Support.Transformer.html#pull/0"},{"type":"function","title":"Grains.Support.Transformer.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.Support.Transformer.html#pull/1"},{"type":"function","title":"Grains.Support.Transformer.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.Support.Transformer.html#pull_with_tag/1"},{"type":"function","title":"Grains.Support.Transformer.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.Support.Transformer.html#pull_with_tag/2"},{"type":"function","title":"Grains.Support.Transformer.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.Support.Transformer.html#push/1"},{"type":"function","title":"Grains.Support.Transformer.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.Support.Transformer.html#push/2"},{"type":"module","title":"Grains.Support.Transformer.State","doc":"","ref":"Grains.Support.Transformer.State.html"},{"type":"module","title":"Grains.Support.Utils","doc":"Small function that help with debugging and testing.","ref":"Grains.Support.Utils.html"},{"type":"function","title":"Grains.Support.Utils.inject_pull/3","doc":"This function takes: the grain supervisor pid short name receiver short name sender It send sends a pull to receiver as if it came from sender . receiver needs to be an existing grain in the recipe. sender can be an existing grain or made up.","ref":"Grains.Support.Utils.html#inject_pull/3"},{"type":"function","title":"Grains.Support.Utils.inject_pull_all/2","doc":"This function takes: the grain supervisor pid short name sender It send sends a pull to predecessors of sender sender needs to be an existing grain in the recipe and alive.","ref":"Grains.Support.Utils.html#inject_pull_all/2"},{"type":"function","title":"Grains.Support.Utils.inject_pull_all_with_tag/3","doc":"Inject pull_with_tag to all predecessors of short_sender","ref":"Grains.Support.Utils.html#inject_pull_all_with_tag/3"},{"type":"function","title":"Grains.Support.Utils.inject_pull_with_tag/4","doc":"Inject pull_with_tag to receiver as if it was issued by short_sender","ref":"Grains.Support.Utils.html#inject_pull_with_tag/4"},{"type":"function","title":"Grains.Support.Utils.inject_push/4","doc":"This function takes: the grain supervisor pid short name receiver short name sender a message msg It send sends a push to receiver with msg as content as if it came from sender . receiver needs to be an existing grain in the recipe. sender can be an existing grain or made up.","ref":"Grains.Support.Utils.html#inject_push/4"},{"type":"module","title":"Grains.Timer","doc":"A periodic timer.","ref":"Grains.Timer.html"},{"type":"function","title":"Grains.Timer.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Grains.Timer.html#child_spec/1"},{"type":"function","title":"Grains.Timer.handle_pull/2","doc":"Callback implementation for Grains.GenGrain.handle_pull/2 .","ref":"Grains.Timer.html#handle_pull/2"},{"type":"function","title":"Grains.Timer.handle_pull/3","doc":"Callback implementation for Grains.GenGrain.handle_pull/3 .","ref":"Grains.Timer.html#handle_pull/3"},{"type":"function","title":"Grains.Timer.handle_push/3","doc":"Callback implementation for Grains.GenGrain.handle_push/3 .","ref":"Grains.Timer.html#handle_push/3"},{"type":"function","title":"Grains.Timer.init/1","doc":"Callback implementation for GenServer.init/1 .","ref":"Grains.Timer.html#init/1"},{"type":"function","title":"Grains.Timer.name/2","doc":"","ref":"Grains.Timer.html#name/2"},{"type":"function","title":"Grains.Timer.own_full_name/0","doc":"See Grains.GenGrain.own_full_name/0 .","ref":"Grains.Timer.html#own_full_name/0"},{"type":"function","title":"Grains.Timer.own_name/0","doc":"See Grains.GenGrain.own_name/0 .","ref":"Grains.Timer.html#own_name/0"},{"type":"function","title":"Grains.Timer.pull/0","doc":"See Grains.GenGrain.pull/0 .","ref":"Grains.Timer.html#pull/0"},{"type":"function","title":"Grains.Timer.pull/1","doc":"See Grains.GenGrain.pull/1 .","ref":"Grains.Timer.html#pull/1"},{"type":"function","title":"Grains.Timer.pull_with_tag/1","doc":"See Grains.GenGrain.pull_with_tag/1 .","ref":"Grains.Timer.html#pull_with_tag/1"},{"type":"function","title":"Grains.Timer.pull_with_tag/2","doc":"See Grains.GenGrain.pull_with_tag/2 .","ref":"Grains.Timer.html#pull_with_tag/2"},{"type":"function","title":"Grains.Timer.push/1","doc":"See Grains.GenGrain.push/1 .","ref":"Grains.Timer.html#push/1"},{"type":"function","title":"Grains.Timer.push/2","doc":"See Grains.GenGrain.push/2 .","ref":"Grains.Timer.html#push/2"},{"type":"module","title":"Grains.Timer.State","doc":"","ref":"Grains.Timer.State.html"},{"type":"extras","title":"Grains - Process Data Flow Orchestration","doc":"Grains - Process Data Flow Orchestration Grains is a framework that uses declarative flow graphs to implement process data flows. graph LR Recipe[Declarative Graph Description] Instantiation[Process Instantiation] Runtime[Runtime Architecture] Recipe --&gt; Runtime Instantiation --&gt; Runtime","ref":"readme.html"},{"type":"extras","title":"Grains - Process Data Flow Orchestration - Motivation","doc":"Implementing data flow graphs with processes can be complicated, as the processes are loosely associated with the implementation. This becomes a problem when the number of processes or the graph becomes large. Grains solves this issue by describing the flow graph in a declarative manner before the processes are started. The declaration makes it visible which processes are connected, and which paths information can take. Since the declarations of the flow graphs are separate from implementation, the structure of the graph is reusable. This allows for systems where the abstractions (structures) stay the same, while the implementation changes. Advantages to an Implicit Data Flow Graph Better system overview Enhance maintainability and testability Increase performance by separating concurrent tasks clearly","ref":"readme.html#motivation"},{"type":"extras","title":"Grains - Process Data Flow Orchestration - Terminology","doc":"Recipe : The declaration of the data flow graph. This is a map from a process to a list of processes which are receiving data from it. Grains : Processes that are part of the data flow graph. Bread : The runtime after combining the recipe with its grains.","ref":"readme.html#terminology"},{"type":"extras","title":"Grains - Process Data Flow Orchestration - Special Processes","doc":"Grains uses special wrapper processes to abstract common functionality into the library. Currently, we support a periodic grain, which pulls data from its predecessor periodically.","ref":"readme.html#special-processes"},{"type":"extras","title":"Grains - Process Data Flow Orchestration - Data Flow Example","doc":"In example/heat/ , a single feedback loop is present as an example for Grains. The task of the example is to hold the temperature within a room at a specified target temperature by using two dedicated heaters. The control system samples the temperature measured at the heaters and activates heating if needed. This is the data flow graph: graph LR subgraph Control System State --&gt;|temperature| Sampler State --&gt;|temperature| Persistence Sampler --&gt;|temperature| UI Sampler --&gt;|temperature| Optimizer end Optimizer --&gt;|command| Heating Heating --&gt;|temperature| State To implement this graph, we define the recipe: recipe = Grains . new_recipe ( :heat , %{ State =&gt; [ Grains . periodic ( Sampler , 1000 ) , Grains . periodic ( Persistence , 500 ) ] , Sampler =&gt; [ UI , Optimizer ] , } ) The recipe contains 5 grains: the system state, a periodic sampler, a periodic persistence process, a UI and an optimizer. State sends information to the sampler and persistence process, while the sampler sends information to the UI and optimizer. We then define how those grains should be started: grains = Grains . new ( %{ State =&gt; { Heat.State , [ ] , [ ] } , Sampler =&gt; { Heat.Sampler , [ ] , [ ] } , Persistence =&gt; { Heat.Persistence , [ ] , [ ] } , Optimizer =&gt; { Heat.Optimizer , [ ] , [ ] } , UI =&gt; { Heat.UI , [ ] , [ ] } } ) This is similar to child specs for supervisors. We map a name used in the recipe to a module with arguments and options. Now we can combine the recipe with the grains to create a running system: { :ok , bread } = Grains . start_supervised ( recipe , grains ) As heaters are not part of the system, they are started outside of this definition: { :ok , _ } = Heating . start_link ( %{ serial : 1 , server : server } ) Example run: mix run - e Heat . run == &gt; grains Compiling 7 files ( . ex ) Generated grains app == &gt; heat Compiling 1 file ( . ex ) Setting target temperature to 23.0 degrees Current average temperature : 19.98333333333335 Current average temperature : 20.166666666666693 Current average temperature : 20.350000000000037 Current average temperature : 20.53333333333338 Current average temperature : 20.716666666666725 Current average temperature : 20.90000000000007 Current average temperature : 21.083333333333414 Current average temperature : 21.266666666666758 Current average temperature : 21.450000000000102 Current average temperature : 21.633333333333447 Current average temperature : 21.81666666666679 Current average temperature : 22.000000000000135","ref":"readme.html#data-flow-example"},{"type":"extras","title":"Grains - Process Data Flow Orchestration - Trivia","doc":"What does a vegetarian zombie eat? Grrrraiiiiiinnnns! License Copyright 2019 sonnen eServices GmbH Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#trivia"},{"type":"extras","title":"Changelog","doc":"Changelog","ref":"changelog.html"},{"type":"extras","title":"Changelog - 1.7.0","doc":"Add optional order parameter for implementation nodes to define child spec order (addresses #37)","ref":"changelog.html#1-7-0"},{"type":"extras","title":"Changelog - 1.6.0","doc":"Add Grains.GenGrain.ghosts_full_name/1 to resolve the name of a ghost (Fix #35, thanks @neosimsim)","ref":"changelog.html#1-6-0"},{"type":"extras","title":"Changelog - 1.5.1","doc":"Fix finding all paths between a start and target node in a bread. This is used for the debugging functions Grains.debug_pull_chain/3 (introduced with v1.5.0) and Grains.debug_reply_chain/3 (introduced with v.1.2.0). The previous implementation missed paths, as the set of visited vertices was global and not per-path.","ref":"changelog.html#1-5-1"},{"type":"extras","title":"Changelog - 1.5.0","doc":"Add Grains.debug_pull_chain/3 to flood pull along all paths from one grain to a transitive predecessor.","ref":"changelog.html#1-5-0"},{"type":"extras","title":"Changelog - 1.4.1","doc":"Fix handle_pull/3 for Grains.Timer and Grains.DebugTimer","ref":"changelog.html#1-4-1"},{"type":"extras","title":"Changelog - 1.4.0","doc":"Add Grains.Support.Constant , which is a grain that pushes a constant value only.","ref":"changelog.html#1-4-0"},{"type":"extras","title":"Changelog - 1.3.1","doc":"Handle pull_with_tag in timers ( Grains.periodic/2,3 ), fix #33.","ref":"changelog.html#1-3-1"},{"type":"extras","title":"Changelog - 1.3.0","doc":"Add Grains.ghost/2,3 to declare a &quot;ghost&quot; edge. This is an edge which is not used with push/pull, but which should be documented in the flow chart.","ref":"changelog.html#1-3-0"},{"type":"extras","title":"Changelog - 1.2.0","doc":"Add GenGrain.debug_reply_chain/3 helper to flush a message on all paths from first_grain to last_grain","ref":"changelog.html#1-2-0"},{"type":"extras","title":"Changelog - 1.1.1","doc":"Fix subscriber field in TestSetup Add bread to TestSetup","ref":"changelog.html#1-1-1"},{"type":"extras","title":"Changelog - 1.1.0","doc":"Grains.Support to help with testing When blackbox-testing a grain, it is common to use a pipeline like this: Grains.Support.Publisher -&gt; GrainUnderTest -&gt; Grains.Support.Subscriber With that setup, GenServer.cast(publisher, message) results in message being pushed into the GrainUnderTest . The messages which are pushed from GrainUnderTest are received by the subscriber and forwarded to the test process. The test process can then use assert_receive . Grains now has helpers to reduce the amount of boilerplate for this setup. Use Grains.Support.setup_test!/2 to set-up the recipe and grains, and inject publish messages through the publisher with Grains.Support.push_into_test/2 .","ref":"changelog.html#1-1-0"},{"type":"extras","title":"Changelog - 1.0.0","doc":"","ref":"changelog.html#1-0-0"},{"type":"extras","title":"Changelog - 0.5.0","doc":"Add Grains.Support.Transformer to transform messages in tests","ref":"changelog.html#0-5-0"},{"type":"extras","title":"Changelog - 0.5.0-rc.3","doc":"Add pull with tags","ref":"changelog.html#0-5-0-rc-3"},{"type":"extras","title":"Changelog - 0.5.0-rc.2","doc":"Small bug fix","ref":"changelog.html#0-5-0-rc-2"},{"type":"extras","title":"Changelog - 0.5.0-rc.1","doc":"Add more options to Grains.Recipe.to_mermaid/2 .","ref":"changelog.html#0-5-0-rc-1"},{"type":"extras","title":"Changelog - 0.5.0-rc.0","doc":"Return error on push/pull when called from a non-grain process Add a sink to eat messages ( Grains.Support.Sink ) Make GenGrain.successors/0 and GenGrain.predecessors/0 public. The functions return an error if called from outside of a grain. Remove default_grains argument from Bread creation Allow Grains.periodic with multiple successors","ref":"changelog.html#0-5-0-rc-0"},{"type":"extras","title":"Changelog - 0.3.7","doc":"Add Grains.route/1,2 function that allows filtering message via pattern","ref":"changelog.html#0-3-7"},{"type":"extras","title":"Changelog - 0.3.6","doc":"Add GenGrain.debug_reply_chain/2 helper to debug asynchronous messages","ref":"changelog.html#0-3-6"},{"type":"extras","title":"Changelog - 0.3.5","doc":"Add Grains.Support.Utils with helper functions to inject push/pull for testing. Add Grains.Support.Trace to trace push/pull messages.","ref":"changelog.html#0-3-5"},{"type":"extras","title":"Changelog - 0.3.4","doc":"Add Grains.get_original_recipe!/1 Add Grains.Recipe.to_mermaid/1,2 to create a mermaid graph from a Recipe.","ref":"changelog.html#0-3-4"},{"type":"extras","title":"Changelog - 0.3.3","doc":"Add pulling through Grains.Support.Subscriber","ref":"changelog.html#0-3-3"},{"type":"extras","title":"Changelog - 0.3.2","doc":"Add pulling through timers Add GenGrain.pull_with_tag/2,3 . This function can be used to add information to a pull operation.","ref":"changelog.html#0-3-2"},{"type":"extras","title":"Changelog - 0.3.1","doc":"Fixes an argument forwarding bug","ref":"changelog.html#0-3-1"},{"type":"extras","title":"Changelog - 0.3.0","doc":"Adds Grains.Support.Cache , a simple caching grain Adds Grains.Support.Publisher , a simple publisher grain Adds Grains.Support.Subscriber , a simple subscriber grain","ref":"changelog.html#0-3-0"},{"type":"extras","title":"Changelog - 0.2.0","doc":"Adds Grains.combine/2 Adds Grains.get_substate/1 Adds Grains.merge/2 to merge two grains. The corresponding function for Grains.Recipe existed since 0.1.0. Adds Grains.Supervisor.which_grains/1 to retrieve the running grains at runtime Implements GenServer return values in GenGrain , making a grain more flexible Adds Grains.Support.DebugTimer to replace the default periodic timer for testing","ref":"changelog.html#0-2-0"},{"type":"extras","title":"Changelog - 0.1.2","doc":"Adds setting a custom id","ref":"changelog.html#0-1-2"},{"type":"extras","title":"Changelog - 0.1.1","doc":"Adds GenGrain.own_full_name/0 and GenGrain.full_name/1","ref":"changelog.html#0-1-1"},{"type":"extras","title":"Changelog - 0.1.0","doc":"Initial version.","ref":"changelog.html#0-1-0"},{"type":"extras","title":"Debugging Message Arrival","doc":"Debugging Message Arrival Working with asynchronous grains can be interesting when certain optional asynchronous behaviour needs to be observed. Consider the following example. Lets assume that A pushes messages to B , and B optionally pushes a messages as a result. How can we determine if B will push a message, without resorting to receive with an infinite timeout? Grains provides a Grains.debug_reply_chain(chain, payload) function to handle this. The function creates a message with a payload , which is passed along the chain grains. As soon as this special message hits the last grain in that chain, the payload is send() to the original caller of Grains.debug_reply_chain/2 . To simplify debugging further, Grains.debug_reply_chain/3 exists, which automatically determines all paths between two grains. With that function, the paths don't need to be written down explicitly.","ref":"debug_message_arrival.html"},{"type":"extras","title":"Debugging Message Arrival - Debugging Pull paths","doc":"As pull/0,1 and pull_with_tag/1,2 also introduce asynchronous behaviour, a debugging function Grains.debug_pull_chain/3 : This function floods messages along all paths from a grain to one of its transitive predecessors, thereby ensuring that all pulls which have been performed before this call have been completed.","ref":"debug_message_arrival.html#debugging-pull-paths"}]